package com.javacorporate.forum.dbobj;

/**
 * ForumSecurity.java
 *
 * Copyright 1999 Javacorporate Ltd.
 */

import com.javacorporate.dbobj.*;
import com.javacorporate.db.DBException;
import java.util.Vector;
import java.util.Enumeration;
import com.javacorporate.common.dbobj.UserGroup;
import com.javacorporate.common.util.LogHandler;

/**
 * Forum Security - what groups are allowed to do what
 *
 * @version		$Revision: 1.1 $  $Date: 2000/05/31 19:41:47 $
 * @author		Michael Nash
 */
public class ForumSecurity extends SecuredDBObject {

	/**
	 * @see com.javacorporate.dbobj.SecuredDBObject#SecuredDBObject
	 */
	public ForumSecurity() throws DBException {
		super();
	} /* ForumSecurity() */
	/**
	 * @see com.javacorporate.dbobj.SecuredDBObject#SecuredDBObject
	 */
	public ForumSecurity(String theUser)
		throws DBException {

		super(theUser);
	} /* ForumSecurity(DBConnection, String) */
	/**
	 * Extends the checkAllRefs method to check for valid UserGroup
	 *
	 * @throws	DBException
	 */
	protected void checkAllRefs() throws DBException {
		checkRef("GroupName", new UserGroup("SYSTEM"), "Invalid " + getDescription("GroupName"));
	} /* checkAllRefs() */
	/**
	 * Implement the standard method to return a new ForumSecurity object
	 *
	 * @returns	DBObject
	 * @throws	DBException
	 */
	public DBObject getThisDBObj() throws DBException {
		return new ForumSecurity();
	} /* getThisDBObj() */
	/**
	 * Override the method getValues to provide specific values for our 
	 * multi-valued fields
	 *
	 * @param	fieldName Fielname to retrieve values for
	 * @returns	Vector of ValidValue Value/description pairs for this field
	 * @throws	DBException If the values cannot be retrieved
	 */
	public synchronized Vector getValidValues(String fieldName)
		throws DBException {

		String myName = new String(thisClass + "getValues(String)");

		if (fieldName.equals("MethodCode")) {
			Vector myValues = new Vector(5);
			myValues.addElement(new ValidValue("A", "Add"));
			myValues.addElement(new ValidValue("U", "Update"));
			myValues.addElement(new ValidValue("D", "Delete"));
			myValues.addElement(new ValidValue("B", "Browse"));
			myValues.addElement(new ValidValue("M", "Moderate"));
			return myValues;
		} else if (fieldName.equals("GroupName")) {
			Vector myValues = new Vector(10);
			ValidValue oneVal;
			UserGroup groupList = new UserGroup("SYSTEM");
			UserGroup oneGroup = null;
			for (Enumeration e = groupList.searchAndRetrieve("Descrip").elements(); e.hasMoreElements(); ) {

				oneGroup = (UserGroup) e.nextElement();
				oneVal = new ValidValue(oneGroup.getField("GroupName"), oneGroup.getField("Descrip"));
				myValues.addElement(oneVal);
			}
			return myValues;
		}
		return null;
	} /* getValidValues(String) */
	/**
	 * @see com.javacorporate.dbobj.SecuredDBObject#SetupFields
	 */
	protected synchronized void setupFields() throws DBException {
		setTargetTable("FORUMSECURITY");

		setName("FORUMSECURITY");
		setDescription("Forum Security");

		addField("GroupId","int", 0, false, "Forum Group Identifier");
		addField("GroupName", "char", 30, false, "Group Name");
		addField("MethodCode", "char", 1, false, "Method Allowed");

		addKey("GroupId");
		addKey("MethodCode");
		addKey("GroupName");

		setMultiValued("GroupName");
		setMultiValued("MethodCode");

		setLookupObject("GroupId", "com.javacorporate.forum.dbobj.ForumGroup");
		
	} /* setupFields() */
} /* ForumSecurity */
