package com.javacorporate.forum.trx;

import java.util.Hashtable;
import java.util.Enumeration;
import com.javacorporate.common.trx.DBTransaction;
import com.javacorporate.common.trx.TrxException;
import com.javacorporate.common.trx.Input;
import com.javacorporate.common.trx.Output;
import com.javacorporate.common.trx.Action;
import com.javacorporate.common.trx.State;
import com.javacorporate.db.DBException;
import com.javacorporate.common.dbobj.User;
import com.javacorporate.forum.dbobj.ForumGroup;
import com.javacorporate.forum.dbobj.ForumThread;
import com.javacorporate.forum.dbobj.ForumMessage;
import com.javacorporate.forum.dbobj.ForumSubscribe;
import java.util.Vector;
import com.javacorporate.common.util.StringUtil;
import com.javacorporate.common.util.LogHandler;
import java.io.StringReader;
import java.io.BufferedReader;
import java.io.IOException;

/**
 * Used from various JSP pages to interact with Forums
 */
public class Browse extends DBTransaction {

	private String thisClass = getClass().getName() + ".";

	public Browse() {
		/* Subscribe to groups (all available groups) */
		State selGroupSubscribe = new State("selGroupSubscribe", "Select Group To Subscribe To");
		addState(selGroupSubscribe);

		/* Prompt for info to start a new group */
		State newGroup = new State("newGroup", "Add New Group");
		newGroup.addParameter("GroupName");
		newGroup.addParameter("GroupDescription");
		addState(newGroup);

		/* Prompt for info to start a new group */
		State promptNewGroup = new State("promptNewGroup", "Prompt for a New Group");
		addState(promptNewGroup);

		/* Subscribe to specified group */
		State promptGroupSubscribe = new State("promptGroupSubscribe", "Prompt for Groups to Subscribe To");
		addState(promptGroupSubscribe);

		/* Subscribe to specified group */
		State groupSubscribe = new State("groupSubscribe","Subscribe To Selected Group");
		addState(groupSubscribe);

		/* Choose a group (from the groups you're subscribed to) */
		State selGroup = new State("selGroup", "Select Group");
		addState(selGroup);
		
		/* Show the threads available in a particular group */
		State browseThreads = new State("browseThreads", "Browse Threads in Group");
		browseThreads.addParameter("GroupId");
		addState(browseThreads);

		/* Show the messages from a particular thread */
		State browseMessages = new State("browseMessages", "Browse Messages in Thread");
		browseMessages.addParameter("GroupId");
		browseMessages.addParameter("ThreadId");
		addState(browseMessages);

		/* Prompt for information to start a new thread */
		State promptNewThread = new State("promptNewThread", "Prompt for New Thread in Group");
		promptNewThread.addParameter("GroupId");
		addState(promptNewThread);

		/* Add a new thread */
		State newThread = new State("newThread", "Add New Thread to Group");
		newThread.addParameter("Subject");
		newThread.addParameter("GroupId");
		addState(newThread);

		/* Add a new message to a thread */
		State newMessage = new State("newMessage","Add New Message in Thread");
		newMessage.addParameter("ThreadId");
		newMessage.addParameter("Subject");
		newMessage.addParameter("Body");
		addState(newMessage);

		/* Prompt for the information for a new message */
		State promptNewMessage = new State("promptNewMessage", "Prompt for New Message in Thread");
		newMessage.addParameter("ThreadId");
		addState(promptNewMessage);

		/* Prompt for the information for a new message */
		State promptReply = new State("promptReply", "Prompt for Reply to a Message");
		promptReply.addParameter("MessageId");
		addState(promptReply);

		/* Reply to an existing message */
		State replyTo = new State("replyTo", "Reply to Message");
		replyTo.addParameter("Subject");
		replyTo.addParameter("Body");
		replyTo.addParameter("MessageId");
		addState(replyTo);
		
	} /* constructor */
/**
 * Insert the method's description here.
 * Creation date: (4/17/00 5:47:46 PM)
 * @param baseMessage com.javacorporate.forum.dbobj.ForumMessage
 * @param messageOutput com.javacorporate.common.trx.Output
 */
private void addReplies(ForumMessage baseMessage, Output messageOutput) throws DBException, IOException {
	ForumMessage replyList = new ForumMessage();
	replyList.setField("ReplyTo", baseMessage.getField("MessageId"));
	ForumMessage oneReply = null;
	for (Enumeration e = replyList.searchAndRetrieve("Created").elements(); e.hasMoreElements(); ) {
		oneReply = (ForumMessage) e.nextElement();

		Output oneMessageOutput = new Output(oneReply.getField("Subject"));

		StringBuffer s = new StringBuffer();
		BufferedReader b = new BufferedReader(new StringReader(oneReply.getField("Body").trim()));

		String oneLine = b.readLine();
		while (oneLine != null) {

			s.append(oneLine);
			s.append("<br>");
			oneLine = b.readLine();
		}
				
		oneMessageOutput.setAttribute("Body",s.toString());
		oneMessageOutput.setAttribute("MessageId",oneReply.getField("MessageId"));
		oneMessageOutput.setAttribute("Created", oneReply.getField("Created"));

		User msgUser = new User("SYSTEM");
		msgUser.setField("UserName", oneReply.getField("UserName"));
		if (msgUser.find()) {
			oneMessageOutput.setAttribute("UserName", msgUser.getField("Descrip"));
		} else {
			oneMessageOutput.setAttribute("UserName", oneReply.getField("UserName"));
		}
				
		messageOutput.addNested(oneMessageOutput);

		addReplies(oneReply, oneMessageOutput);
	}	
}
	/**
	 * Browse messages in a thread
	 */
	private void browseMessagesState() throws TrxException {
		String myName = new String(thisClass + "browseMessages()");

		try {
			ForumGroup thisGroup = new ForumGroup("SYSTEM");
			thisGroup.setField("GroupId", getParameter("GroupId"));
			thisGroup.retrieve();
			if (!thisGroup.allowedForUser(getUser(), "B")) {
				throw new SecurityException(myName + ":User '" + getUser() + "' is not allowed to browse messages "
					+ " in group " + thisGroup.getField("GroupId"));
			}

			addOutput(new Output("GroupDescription", thisGroup.getField("GroupDescription")));
			addOutput(new Output("GroupId", getParameter("GroupId")));

			ForumThread thisThread = new ForumThread("SYSTEM");
			thisThread.setField("ThreadId", getParameter("ThreadId"));
			thisThread.retrieve();

			addOutput(new Output("ThreadSubject", thisThread.getField("Subject")));
			addOutput(new Output("ThreadId", getParameter("ThreadId")));

			Output messages = new Output("messages", "messages");
			addOutput(messages);

			ForumMessage ml = new ForumMessage("SYSTEM");
			ml.setField("ThreadId", getParameter("ThreadId"));
			ml.setField("ReplyTo", "0");
			ForumMessage oneMessage = null;

			Output oneMessageOutput = null;
			Output oneMessageId = null;

			for (Enumeration e = ml.searchAndRetrieve("Created DESC").elements(); e.hasMoreElements(); ) {
				oneMessage = (ForumMessage) e.nextElement();

				oneMessageOutput = new Output(oneMessage.getField("Subject"));

				StringBuffer s = new StringBuffer();
				BufferedReader b = new BufferedReader(new StringReader(oneMessage.getField("Body").trim()));

				String oneLine = b.readLine();
				while (oneLine != null) {

					s.append(oneLine);
					s.append("<br>");
					oneLine = b.readLine();
				}
				
				oneMessageOutput.setAttribute("Body", s.toString());

				
				oneMessageOutput.setAttribute("MessageId",oneMessage.getField("MessageId"));
				oneMessageOutput.setAttribute("Created", oneMessage.getField("Created"));

				User msgUser = new User("SYSTEM");
				msgUser.setField("UserName", oneMessage.getField("UserName"));

				/* Use the lines comment out below to show full name (if available) */
				//if (msgUser.find()) {
				//	oneMessageOutput.setAttribute("UserName", msgUser.getField("Descrip"));
				//} else {
					oneMessageOutput.setAttribute("UserName", oneMessage.getField("UserName"));
				//}

				messages.addNested(oneMessageOutput);

				/* Now see if there are any messages in reply to this message */
				addReplies(oneMessage, oneMessageOutput);
			}
		} catch(DBException de) {
			throw new TrxException(myName + ":Unable to retrieve messages", de);
		} catch (IOException ie) {
			throw new TrxException(myName + ":Unable to parse message", ie);
		}
	} /* browseMessages() */
	/**
	 * Allow user to choose a thread to browse
	 */
	private void browseThreadsState() throws TrxException {
		String myName = new String(thisClass + "browseThreadsState()");

		try {
			User myUser = new User();
			myUser.setField("UserName", getUser());
			myUser.retrieve();
			addOutput(new Output("UserDescrip", myUser.getField("Descrip")));

			ForumGroup thisGroup = new ForumGroup("SYSTEM");
			thisGroup.setField("GroupId", getParameter("GroupId"));
			thisGroup.retrieve();
			if (!thisGroup.allowedForUser(getUser(), "B")) {
				throw new SecurityException(myName + ":User '" + getUser() + "' is not allowed to browse threads "
					+ " in group " + thisGroup.getField("GroupId"));
			}


			Output gd = new Output("GroupDescription", thisGroup.getField("GroupDescription"));
			addOutput(gd);

			/* Output "Threads" contains each of the threads in this group */
			/* as nested items */
			Output th = new Output("Threads", "Threads");
			addOutput(th);

			addOutput(new Output("GroupId", getParameter("GroupId")));

			ForumThread tl = new ForumThread("SYSTEM");
			tl.setField("GroupId", getParameter("GroupId"));
			ForumThread oneThread = null;

			Output oneThreadOutput = null;
			Output oneThreadId = null;

			for (Enumeration e = tl.searchAndRetrieve().elements(); 
				e.hasMoreElements(); ) {
				oneThread = (ForumThread) e.nextElement();

				oneThreadOutput = new Output(oneThread.getField("Subject"));

				oneThreadId = new Output(oneThread.getField("ThreadId"));
				oneThreadId.setName("ThreadId");
				oneThreadOutput.addNested(oneThreadId);

				oneThreadOutput.addNested(new Output("LastPosting", oneThread.getField("LastPosting")));
				oneThreadOutput.addNested(new Output("LastPostingBy", oneThread.getField("LastPostingBy")));

				ForumMessage fm = new ForumMessage();
				fm.setField("ThreadId", oneThread.getField("ThreadId"));
				oneThreadOutput.addNested(new Output("Count", "" + fm.count()));

				th.addNested(oneThreadOutput);
			}
		} catch(DBException de) {
			throw new TrxException(myName + ":Unable to retrieve group information:" + de.getMessage());
		} 
	} /* browseThreadsState() */
	/**
	 * Return the title of this Transaction 
	 */
	public String getTitle() {
		return new String("Browse Forum");
	} /* getTitle() */
/**
 * Subscribe to the selected groups
 */
private void groupSubscribeState() throws TrxException {
	Hashtable params = getParameters();
	addOutput(new Output("Subscriptions Updated"));
	try {
		ForumGroup gl = new ForumGroup("SYSTEM");
		ForumGroup oneGroup = null;
		ForumSubscribe oneSubscribe = new ForumSubscribe();
		for (Enumeration e = gl.searchAndRetrieve().elements(); e.hasMoreElements();) {
			oneGroup = (ForumGroup) e.nextElement();
			if (oneGroup.allowedForUser(getUser(), "B")) {
				oneSubscribe.clear();
				oneSubscribe.setField("GroupId", oneGroup.getField("GroupId"));
				oneSubscribe.setField("UserName", getUser());
				if (params.containsKey("Subscribe_" + oneGroup.getField("GroupId"))) {
					if (!oneSubscribe.find()) {
						if (params.containsKey("EMail_" + oneGroup.getField("GroupId"))) {
							oneSubscribe.setField("EMail", "Y");
						}
						oneSubscribe.add();
						addOutput(new Output("Added subscription to group " + oneGroup.getField("GroupName")));
					} else {
						if (params.containsKey("EMail_" + oneGroup.getField("GroupId"))) {
							if (!oneSubscribe.getField("EMail").equals("Y")) {
								oneSubscribe.setField("EMail", "Y");
								oneSubscribe.update();
								addOutput(new Output("Added email message services for group " + oneGroup.getField("GroupName")));
							}
						} else {
							if (oneSubscribe.getField("EMail").equals("Y")) {
								oneSubscribe.setField("EMail", "");
								oneSubscribe.update();
								addOutput(new Output("Removed email message services for group " + oneGroup.getField("GroupName")));
							}
						}	
					}
				} else {
					if (oneSubscribe.find()) {
						oneSubscribe.delete();
						addOutput(new Output("Removed subscription to group " + oneGroup.getField("GroupName")));
					}
				}
			}
		}
		Action selGrp = new Action("Select Group", getClass().getName());
		selGrp.setName("selGroup");
		selGrp.addParam("state", "selGroup");
		selGrp.addParam("next", "/components/forum/jsp/groups.jsp");
		addAction(selGrp);
	} catch (DBException de) {
		throw new TrxException(de.getMessage());
	}
}
/* groupSubscribeState() */
	/**
	 * Create a new group
	 */
	private void newGroupState() throws TrxException {
		String myName = new String(thisClass + "newGroupState()");

		try {
			ForumGroup newGroup = new ForumGroup(getUser());
			newGroup.setField("GroupName", getParameter("GroupName"));
			newGroup.setField("GroupDescription", getParameter("GroupDescription"));
			newGroup.add();
			//addOutput(new Output("GroupId", newGroup.getField("GroupId")));
			addOutput(new Output("New Group Created (Id is " + newGroup.getField("GroupId")));

			Action ag = new Action("Browse New Group", "com.javacorporate.forum.trx.Browse");
			ag.addParam("state", "browseThreads");
			ag.addParam("GroupId", newGroup.getField("GroupId"));
			ag.addParam("next", "/components/forum/jsp/threads.jsp");
			addAction(ag);
			
		} catch(DBException de) {
			throw new TrxException(myName + ":Unable to create new group:" + de.getMessage());
		} 
	} /* newGroupState() */
	/**
	 * Create a new message in the given thread (not a reply)
	 */
	private void newMessageState() throws TrxException {
		String myName = new String(thisClass + "newMessageState()");

		try {
			ForumThread ft = new ForumThread("SYSTEM");
			ft.setField("ThreadId", getParameter("ThreadId"));
			ft.retrieve();
			
			ForumGroup fg = new ForumGroup("SYSTEM");
			fg.setField("GroupId", ft.getField("GroupId"));
			addOutput(new Output("GroupId", ft.getField("GroupId")));
			fg.retrieve();

			if (!fg.allowedForUser(getUser(), "A")) {
				throw new SecurityException(myName + ":User '" + getUser() + "' is not allowed to add messages "
					+ " in group " + fg.getField("GroupId"));
			}
			
			ForumMessage fm = new ForumMessage("SYSTEM");
			fm.setField("ThreadId", getParameter("ThreadId"));
			fm.setField("Subject", getParameter("Subject"));
			fm.setField("Body", getParameter("Body"));
			fm.setField("Status", "0");
			fm.setField("UserName", getUser());
			fm.setField("NextMessage", "0");
			fm.add();
			addOutput(new Output("ThreadSubject", getParameter("Subject")));
			addOutput(new Output("ThreadId", getParameter("ThreadId")));

	
			addOutput(new Output("GroupDescription", fg.getField("GroupDescription")));

			Action ag = new Action("Return to Thread", "com.javacorporate.forum.trx.Browse");
			ag.addParam("state", "browseMessages");
			ag.addParam("ThreadId", getParameter("ThreadId"));
					ag.addParam("next", "/components/forum/jsp/messages.jsp");
			addAction(ag);
			
		} catch (DBException de) {
			throw new TrxException(myName + ":Unable to store new message", de);
		}
	} /* newMessageState() */
	/**
	 * Transition to the new state
	 */
	public void newState(String newState) throws TrxException {
		super.newState(newState);

		String myName = new String(thisClass + "newState(String)");

		if (newState.equals("selGroup")) {
			selGroupState();
		} else if (newState.equals("browseThreads")) {
			browseThreadsState();
		} else if (newState.equals("browseMessages")) {
			browseMessagesState();
		} else if (newState.equals("promptNewMessage")) {
			promptNewMessageState();
		} else if (newState.equals("newGroup")) {
			newGroupState();
		} else if (newState.equals("newMessage")) {
			newMessageState();
		} else if (newState.equals("newThread")) {
			newThreadState();
		} else if (newState.equals("promptNewThread")) {
			promptNewThreadState();
		} else if (newState.equals("promptReply")) {
			promptReplyState();
		} else if (newState.equals("replyTo")) {
			replyToState();
		} else if (newState.equals("promptGroupSubscribe")) {
			promptGroupSubscribeState();
		} else if (newState.equals("groupSubscribe")) {
			groupSubscribeState();
		} else if (newState.equals("promptNewGroup")) {
			promptNewGroupState();
		} else {
			throw new TrxException(myName + ":Unknown state '" + newState + "'");
		}		
	} /* newState(String) */
	/**
	 * Create a new thread in the given group
	 */
	private void newThreadState() throws TrxException {
		String myName = new String(thisClass + "newThreadState()");

		try {
			ForumThread newThread = new ForumThread("SYSTEM");
			newThread.setField("Subject", getParameter("Subject"));
			newThread.setField("GroupId", getParameter("GroupId"));

			ForumGroup fg = new ForumGroup();
			fg.setField("GroupId", getParameter("GroupId"));
			fg.retrieve();

			if (!fg.allowedForUser(getUser(), "A")) {
				throw new SecurityException(myName + ":User '" + getUser() + "' is not allowed to add messages "
					+ " in group " + fg.getField("GroupId"));
			}

			newThread.add();
		
			addOutput(new Output("New Thread Created"));

			Action ag = new Action("Return to Group", "com.javacorporate.forum.trx.Browse");
			ag.addParam("state", "browseThreads");
			ag.addParam("GroupId", fg.getField("GroupId"));
			ag.addParam("next", "/components/forum/jsp/threads.jsp");
			addAction(ag);

			
		} catch(DBException de) {
			throw new TrxException(myName + ":Unable to create new thread", de);
		} 
	} /* newThreadState() */
/**
 * Prompt the user for groups to subscribe to
 */
private void promptGroupSubscribeState() throws TrxException {
	String myName = new String(thisClass + "promptGroupSubscribeState()");
	try {
		Output groups = new Output("groups", "Available Groups");
		addOutput(groups);
		ForumGroup gl = new ForumGroup("SYSTEM");
		ForumGroup oneGroup = null;
		for (Enumeration e = gl.searchAndRetrieve().elements(); e.hasMoreElements();) {
			oneGroup = (ForumGroup) e.nextElement();
			if (oneGroup.allowedForUser(getUser(), "B")) {
				Output o = new Output(oneGroup.getField("GroupDescription"));
				groups.addNested(o);
				Output group = new Output(oneGroup.getField("GroupId"));
				group.setName("GroupId");
				o.addNested(group);
				ForumThread ft = new ForumThread();
				ft.setField("GroupId", oneGroup.getField("GroupId"));
				o.addNested(new Output("Count", "" + ft.count()));
				ForumSubscribe fs = new ForumSubscribe();
				fs.setField("GroupId", oneGroup.getField("GroupId"));
				o.addNested(new Output("SubscriberCount", "" + fs.count()));
				fs.clear();
				fs.setField("GroupId", oneGroup.getField("GroupId"));
				fs.setField("UserName", getUser());
				if (fs.find()) {
					o.addNested(new Output("Subscribe_" + oneGroup.getField("GroupId"), "CHECKED"));
					if (fs.getField("EMail").equalsIgnoreCase("Y")) {
						o.addNested(new Output("EMail_" + oneGroup.getField("GroupId"), "CHECKED"));
					} else {
						o.addNested(new Output("EMail_" + oneGroup.getField("GroupId"), ""));
					}
				} else {
					o.addNested(new Output("Subscribe_" + oneGroup.getField("GroupId"), ""));
					o.addNested(new Output("EMail_" + oneGroup.getField("GroupId"), ""));
				}
			}
		}
	} catch (DBException de) {
		throw new TrxException(myName + ":Unable to retrieve group information:" + de.getMessage());
	}
}
/* promptGroupSubscribeState() */
	/**
	 * Prompt the user for the info needed to create a new group
	 */
	private void promptNewGroupState() throws TrxException {

		Input g1 = new Input("GroupName");
		g1.setLabel("Name");
		addInput(g1);

		Input g2 = new Input("GroupDescription");
		g2.setLabel("Description");
		addInput(g2);

	} /* promptNewGroupState() */
	/**
	 * Prompt for a new message in this thread
	 */	
	private void promptNewMessageState() throws TrxException {
		String myName = new String(thisClass + "promptNewMessageState()");

		addOutput(new Output("GroupId", getParameter("GroupId")));
		addOutput(new Output("ThreadId", getParameter("ThreadId")));

		System.out.println("About to prompt for new message");
		
		try {
			ForumThread ft = new ForumThread("SYSTEM");
			ft.setField("ThreadId", getParameter("ThreadId"));
			ft.retrieve();

			ForumGroup fg = new ForumGroup();
			fg.setField("GroupId",ft.getField("GroupId"));
			fg.retrieve();

			if (!fg.allowedForUser(getUser(), "A")) {
				throw new SecurityException(myName + ":User '" + getUser() + "' is not allowed to add messages "
					+ " in group " + fg.getField("GroupId"));
			}
			
			Input subject = new Input("Subject");
			subject.setDefaultValue("");
			addInput(subject);	
		} catch(DBException de) {
			throw new TrxException(myName + ":" + de.getMessage());
		}

		Input body = new Input("Body");
		body.setDefaultValue("");
		addInput(body);
	} /* promptNewMessageState() */
	/**
	 * Prompt for a new thread
	 */
	private void promptNewThreadState() throws TrxException {
		String myName = new String(thisClass + "promptNewThreadState()");
		
		addOutput(new Output("GroupId", getParameter("GroupId")));

		try {
			ForumGroup fg = new ForumGroup();
			fg.setField("GroupId", getParameter("GroupId"));
			fg.retrieve();

			if (!fg.allowedForUser(getUser(), "A")) {
				throw new SecurityException(myName + ":User '" + getUser() + "' is not allowed to add threads "
					+ " in group " + fg.getField("GroupId"));
			}
		} catch (DBException de) {
			throw new TrxException(de);
		}
			
		Input subject = new Input("Subject");
		subject.setLabel("Subject");
		subject.setDefaultValue("");
		addInput(subject);
	} /* promptNewThreadState() */
	/**
	 * Prompt for a reply to this message
	 */	
	private void promptReplyState() throws TrxException {
		String myName = new String(thisClass + "promptNewMessageState()");

		//addOutput(new Output("GroupId", getParameter("GroupId")));
		//addOutput(new Output("ThreadId", getParameter("ThreadId")));
		try {
			ForumMessage originalMessage = new ForumMessage();
			originalMessage.setField("MessageId", getParameter("MessageId"));
			originalMessage.retrieve();

			ForumThread ft = new ForumThread();
			ft.setField("ThreadId", originalMessage.getField("ThreadId"));
			ft.retrieve();

			ForumGroup fg = new ForumGroup();
			fg.setField("GroupId", ft.getField("GroupId"));
			fg.retrieve();

			if (!fg.allowedForUser(getUser(), "A")) {
				throw new SecurityException(myName + ":User '" + getUser() + "' is not allowed to add messages "
					+ " in group " + fg.getField("GroupId"));
			}

			Input subject = new Input("Subject");
			subject.setDefaultValue("RE:" + originalMessage.getField("Subject"));
			addInput(subject);

		} catch(DBException de) {
			throw new TrxException(myName + ":" + de.getMessage());
		}

		Input body = new Input("Body");
		body.setDefaultValue("");
		addInput(body);
		
		
	} /* promptReplyState() */
	/**
	 * Reply to an existing message
	 */
	private void replyToState() throws TrxException {
		String myName = new String(thisClass + "replyToState()");

		try {
			ForumMessage originalMessage = new ForumMessage();
			originalMessage.setField("MessageId", getParameter("MessageId"));
			originalMessage.retrieve();
			
			ForumMessage fm = new ForumMessage();

			fm.setField("Subject", getParameter("Subject"));
			fm.setField("Body", getParameter("Body"));
			fm.setField("Status", "0");
			fm.setField("UserName", getUser());
			fm.setField("ReplyTo", originalMessage.getField("MessageId"));
			fm.setField("ThreadId", originalMessage.getField("ThreadId"));
			addOutput(new Output("ThreadId", originalMessage.getField("ThreadId")));

			ForumThread myThread = new ForumThread();
			myThread.setField("ThreadId", originalMessage.getField("ThreadId"));
			myThread.retrieve();
			addOutput(new Output("GroupId", myThread.getField("GroupId")));

			ForumGroup fg = new ForumGroup();
			fg.setField("GroupId", myThread.getField("GroupId"));
			fg.retrieve();

			if (!fg.allowedForUser(getUser(), "A")) {
				throw new SecurityException(myName + ":User '" + getUser() + "' is not allowed to reply to messages "
					+ " in group " + fg.getField("GroupId"));
			}
	
			fm.add();
			addOutput(new Output("MessageId", fm.getField("MessageId")));

			Action ag = new Action("Return to Thread", "com.javacorporate.forum.trx.Browse");
			ag.addParam("state", "browseMessages");
			ag.addParam("ThreadId", originalMessage.getField("ThreadId"));
			ag.addParam("next", "/components/forum/jsp/threads.jsp");
			addAction(ag);
		} catch (DBException de) {
			throw new TrxException(de);
		}

	} /* replyToState() */
	/**
	 * Allow user to choose a group 
	 */
	private void selGroupState() throws TrxException {
		String myName = new String(thisClass + "selGroupState()");

		try {
			User myUser = new User();
			myUser.setField("UserName", getUser());
			myUser.retrieve();
			addOutput(new Output("UserDescrip", myUser.getField("Descrip")));

			Output groups = new Output("groups", "Subscribed Groups");
			addOutput(groups);

			ForumSubscribe fl = new ForumSubscribe("SYSTEM");
			fl.setField("UserName", getUser());
			ForumSubscribe oneSubscribe = null;

			String projectCode = StringUtil.notNull(getParameter("Project"));
			if (!projectCode.equals("")) {
				addOutput(new Output("Project", projectCode));
			}

			ForumGroup oneGroup = new ForumGroup();
			for (Enumeration e = fl.searchAndRetrieve().elements(); e.hasMoreElements(); ) {

				oneSubscribe = (ForumSubscribe) e.nextElement();
				
				oneGroup.clear();
				oneGroup.setField("GroupId", oneSubscribe.getField("GroupId"));
				oneGroup.retrieve();

				if (oneGroup.allowedForUser(getUser(), "B")) {

					if (projectCode.equals("") || (projectCode.equals(oneGroup.getField("Project")))) {
				
						Output o = new Output(oneGroup.getField("GroupDescription"));

						groups.addNested(o);
						Output group = new Output(oneGroup.getField("GroupId"));
						group.setName("GroupId");
						o.addNested(group);

						ForumThread ft = new ForumThread();
						ft.setField("GroupId", oneGroup.getField("GroupId"));
						o.addNested(new Output("Count", "" + ft.count()));

						ForumSubscribe fs = new ForumSubscribe();
						fs.setField("GroupId", oneGroup.getField("GroupId"));
						o.addNested(new Output("SubscriberCount", "" + fs.count()));
					}
				} else {
					LogHandler.tryLog(myName, "WARNING: User '" + getUser() + "' is subscribed to forum group "
						+ oneGroup.getField("GroupId") + ", but does not have permission to browse.", "YELLOW");
				}
			}
		} catch(DBException de) {
			throw new TrxException(myName + ":Unable to retrieve "
				+ "group information:" + de.getMessage());
		} 

	} /* selGroupState() */
} /* Browse */
